@@ -1,3 +1,4 @@
+from typing import Union
 from pygl_nf.GL_MATH import Vec2_, Math_
 from pygl_nf import GL_MATH
 import random 
@@ -142,9 +143,27 @@ class Particles():
                         rotating_gravity_angle = 0,
                         image_rotate_in_position = False,
                         image_rotate_in_vector = False,
-                        image_rotate_in_position_angle = 0
+                        image_rotate_in_position_angle = 0,
+                        teni = False,
+                        teni_vector = [1,1],
+                        teni_color = (0,0,0),
+                        teni_fill = True,
+                        teni_size_obvodka = 5,
+                        size_randoming = False,
+                        size_min_max = [],
+                        size_resize_timer = 1,
+                        vector_speed_randoming_angle = 90
+                    
                         ) -> None:
-                
+                self.vector_speed_randoming_angle = vector_speed_randoming_angle
+                self.size_resize_timer = size_resize_timer
+                self.size_randoming = size_randoming
+                self.size_min_max = size_min_max
+                self.teni_size_obvodka = teni_size_obvodka
+                self.teni_fill = teni_fill
+                self.teni_color = teni_color
+                self.vector_teni = teni_vector
+                self.teni = teni
                 self.start_render = True
                 self.start_emit = True
                 self.start_focus = True
@@ -201,6 +220,10 @@ class Particles():
                 self.randoming_image = randomig_iamge
                 self.end_mouse_pos = 0
 
+
+                self.phisics_color = None
+
+
                 self.shape_data = shape_data
                 self.shape = self.shape_data[0]
                 self.shape_color = self.shape_data[2]
@@ -223,7 +246,38 @@ class Particles():
 
 
                 self.id = random.randint(1,1000000000)
-                
+        def Phisics(self,color,randoming_vector,vector=[10,10],tyga=[0,1],density_y=0.9,density_x=0.6):
+            self.phisics_color = color
+            tyga = GL_MATH.Vec2_(tyga)
+            
+            for i in range(len(self.particles)):
+                self.particles[i][1].vector+=tyga.vector
+                if self.shape == 'c':
+                    if (self.surf.IN_WINDOW([int(self.particles[i][0].vector.x+self.particles[i][2]+self.particles[i][1].vector.x), int(self.particles[i][0].vector.y+self.particles[i][1].vector.y)])  and    
+                        self.surf.IN_WINDOW([int(self.particles[i][0].vector.x-self.particles[i][2]+self.particles[i][1].vector.x), int(self.particles[i][0].vector.y+self.particles[i][1].vector.y)])  and
+                        self.surf.IN_WINDOW([int(self.particles[i][0].vector.x+self.particles[i][1].vector.x), int(self.particles[i][0].vector.y+self.particles[i][2]+self.particles[i][1].vector.y)])  and
+                        self.surf.IN_WINDOW([int(self.particles[i][0].vector.x+self.particles[i][1].vector.x), int(self.particles[i][0].vector.y-self.particles[i][2]+self.particles[i][1].vector.y)])     ):
+
+                        if randoming_vector==False:
+                            if self.surf.GET_COLOR( int(self.particles[i][0].vector.x+self.particles[i][1].vector.x), int(self.particles[i][0].vector.y+self.particles[i][2]+self.particles[i][1].vector.y) ) == color or self.surf.GET_COLOR( int(self.particles[i][0].vector.x+self.particles[i][1].vector.x), int(self.particles[i][0].vector.y-self.particles[i][2]+self.particles[i][1].vector.y) ) == color:
+                                self.particles[i][1].vector.y = -(self.particles[i][1].vector.y*density_y)
+                                self.particles[i][1].vector.x = (self.particles[i][1].vector.x*density_x)
+
+                            if self.surf.GET_COLOR( int(self.particles[i][0].vector.x+self.particles[i][2]+self.particles[i][1].vector.x), int(self.particles[i][0].vector.y+self.particles[i][1].vector.y) ) == color or self.surf.GET_COLOR( int(self.particles[i][0].vector.x-self.particles[i][2]+self.particles[i][1].vector.x), int(self.particles[i][0].vector.y+self.particles[i][1].vector.y) ) == color:
+                                self.particles[i][1].vector.x = -(self.particles[i][1].vector.x*density_x)
+                            
+                        
+                                
+                        else:
+                            if self.surf.GET_COLOR( int(self.particles[i][0].vector.x+self.particles[i][1].vector.x), int(self.particles[i][0].vector.y+self.particles[i][2]+self.particles[i][1].vector.y) ) == color or self.surf.GET_COLOR( int(self.particles[i][0].vector.x+self.particles[i][1].vector.x), int(self.particles[i][0].vector.y-self.particles[i][2]+self.particles[i][1].vector.y) ) == color:
+                                self.particles[i][1].vector.y = -(self.particles[i][1].vector.y*density_y)
+                                self.particles[i][1].vector.x += random.randint(-vector[0],vector[0])
+
+                            if self.surf.GET_COLOR( int(self.particles[i][0].vector.x+self.particles[i][2]+self.particles[i][1].vector.x), int(self.particles[i][0].vector.y+self.particles[i][1].vector.y) ) == color or self.surf.GET_COLOR( int(self.particles[i][0].vector.x-self.particles[i][2]+self.particles[i][1].vector.x), int(self.particles[i][0].vector.y+self.particles[i][1].vector.y) ) == color:
+                                self.particles[i][1].vector.x = -(self.particles[i][1].vector.x*density_x)
+                                self.particles[i][1].vector.y += random.randint(-vector[1],vector[1])
+            
+
         
         def mult(self,num):
             if num<=0:
@@ -243,59 +297,77 @@ class Particles():
                 self.time+=self.spawn_delta
                 if self.time%self.spawn_time==0:
                     self.end_line_pos = [self.posx,self.posy]
-                    for i in range(self.partcle_count):
-                        pos = Vec2_([
-                            random.randint(self.posx,self.posx+self.width),
-                            random.randint(self.posy,self.posy+self.height)
-                        ])
-                        
-                        self.cirkul_spee = Vec2_([
-                            random.randint(-10000,10000)/10000*self.circle_speed,
-                            random.randint(-10000,10000)/10000*self.circle_speed
-                        ])
-                        
-                        self.vector_spee = Vec2_(self.vector_speed)
-                        
-                        if self.shape != 'i':
-                            if self.color_randoming == True and type(self.shape_color)==list and len(self.shape_color)>1:
-                                color = self.shape_color[random.randint(0,len(self.shape_color)-1)]
-                            else:
-                                color = self.shape_color[self.color_index]
-                            if self.shape_color == 'r':
-                                color =Color_._rgb('r','r','r').COLOR
-                            particle = [pos,self.cirkul_spee,self.shape_widt,self.vector_spee,self.life_time,color,0]
-                                        
-                        else:
-                            color = None
-                            if self.image_rotating:
-                                rotate = random.randint(-self.image_rotating_delta,self.image_rotating_delta)
-                            else:
-                                rotate = 0
-                            
-                            if self.randoming_image:
-                                ri = random.randint(0,len(self.img)-1)
-                                img = self.img[ri]
+                    self.posx = int(self.posx) ; self.posy = int(self.posy)
+                    if self.surf.IN_WINDOW([self.posx,self.posy]):
+                        if self.surf.GET_COLOR(self.posx,self.posy)!=self.phisics_color:
+                            for i in range(self.partcle_count):
+                                pos = Vec2_([
+                                    random.randint(self.posx,self.posx+self.width),
+                                    random.randint(self.posy,self.posy+self.height)
+                                ])
                                 
-                                rect = self.img[ri].img.get_rect(center = pos.vector.xy)
-                                particle = [pos,self.cirkul_spee,self.shape_widt,self.vector_spee,self.life_time,color,rotate,0,rect,img]
-                            else:
-                                rect = self.img.img.get_rect(center = pos.vector.xy)
-                                particle = [pos,self.cirkul_spee,self.shape_widt,self.vector_spee,self.life_time,color,rotate,0,rect]
+                                self.cirkul_spee = Vec2_([
+                                    random.randint(-10000,10000)/10000*self.circle_speed,
+                                    random.randint(-10000,10000)/10000*self.circle_speed
+                                ])
+                                
+                                self.vector_spee = Vec2_(self.vector_speed)
+                                angle = random.randint(-self.vector_speed_randoming_angle,self.vector_speed_randoming_angle)
+                                self.vector_spee.Rotate(angle)
+                                
+                                if self.shape != 'i':
+                                    if self.color_randoming == True and type(self.shape_color)==list and len(self.shape_color)>1:
+                                        color = self.shape_color[random.randint(0,len(self.shape_color)-1)]
+                                    else:
+                                        color = self.shape_color[self.color_index]
+                                    if self.shape_color == 'r':
+                                        color =Color_._rgb('r','r','r').COLOR
+                                    if self.size_randoming:
+                                        self.shape_widt = random.randint(self.size_min_max[0],self.size_min_max[1])
+                                    particle = [pos,self.cirkul_spee,self.shape_widt,self.vector_spee,self.life_time,color,0,self.size_resize_timer]
+                                                
+                                else:
+                                    color = None
+                                    if self.image_rotating:
+                                        rotate = random.randint(-self.image_rotating_delta,self.image_rotating_delta)
+                                    else:
+                                        rotate = 0
+                                    
+                                    if self.randoming_image:
+                                        ri = random.randint(0,len(self.img)-1)
+                                        img = self.img[ri]
+                                        
+                                        rect = self.img[ri].img.get_rect(center = pos.vector.xy)
+                                        if self.size_randoming:
+                                            self.shape_widt = random.randint(self.size_min_max[0],self.size_min_max[1])
+                                        particle = [pos,self.cirkul_spee,self.shape_widt,self.vector_spee,self.life_time,color,rotate,0,rect,img]
+                                    else:
+                                        rect = self.img.img.get_rect(center = pos.vector.xy)
+                                        if self.size_randoming:
+                                            self.shape_widt = random.randint(self.size_min_max[0],self.size_min_max[1])
+                                        particle = [pos,self.cirkul_spee,self.shape_widt,self.vector_spee,self.life_time,color,rotate,0,rect]
 
-                        
+                                
 
-                        self.particles.append(particle)
+                                self.particles.append(particle)
 
         def Render(self):
             if self.start_render:
+                if self.teni:
+                    for i in range(len(self.particles)):
+                        if self.shape == 'c':
+                            if self.teni_fill:
+                                self.surf.GL.Circle(self.teni_color,[self.particles[i][0].vector.x+self.vector_teni[0],self.particles[i][0].vector.y+self.vector_teni[1]],self.particles[i][2],0,'s','D')
+                            else:
+                                self.surf.GL.Circle(self.teni_color,[self.particles[i][0].vector.x+self.vector_teni[0],self.particles[i][0].vector.y+self.vector_teni[1]],self.particles[i][2],self.teni_size_obvodka,'s','D')
+
+                        elif self.shape == 'r':
+                            if self.teni_fill:
+                                self.surf.GL.Rect(self.teni_color,[self.particles[i][0].vector.x-self.particles[i][2]/2+self.vector_teni[0],self.particles[i][0].vector.y-self.particles[i][2]/2+self.vector_teni[1]],[self.particles[i][2],self.particles[i][2]],0,'s','D')
+
+
                 for i in range(len(self.particles)):
 
-                    
-
-
-
-
-
                     if self.shape == 'r':
 
                         self.surf.GL.Rect(self.particles[i][5],[self.particles[i][0].vector.x-self.particles[i][2]/2,self.particles[i][0].vector.y-self.particles[i][2]/2],[self.particles[i][2],self.particles[i][2]],0,'s','D')
@@ -353,14 +425,19 @@ class Particles():
                 if self.rotate_gravity:
                     self.gravity.Rotate(self.rotating_gravity_angle)
                 for i in range(len(self.particles)):
+
+                    if self.size_resize_timer!=None:
+                        self.particles[i][7]-=1
+
+
                     if self.rotate_in_center:
                         self.particles[i][1].Rotate(self.rotating_in_center_angle)
                     
                     self.particles[i][1].vector += self.gravity.vector
 
                     if self.move_point_set:    
-                        sx = -(self.particles[i][0].pos1[0]-self.move_point[0])/self.move_point_speed
-                        sy = -(self.particles[i][0].pos1[1]-self.move_point[1])/self.move_point_speed
+                        sx = -(self.particles[i][0].vector.x-self.move_point[0])/self.move_point_speed
+                        sy = -(self.particles[i][0].vector.y-self.move_point[1])/self.move_point_speed
                         if GL_MATH.Math_().RAST(self.particles[i][0].vector.xy,self.move_point)<self.move_radius:
                             sx*=self.move_radius_delta_x
                             sy*=self.move_radius_delta_y
@@ -376,12 +453,13 @@ class Particles():
                     self.particles[i][0].vector += self.particles[i][1].vector
 
                     self.particles[i][4]-=self.life_delta
-                    if self.size_resize:
-                        if self.return_size:  
-                            if self.particles[i][2]<=self.shape_max_size+10:
-                                self.particles[i][2]+=self.size_deller
-                        else:
-                            self.particles[i][2]-=self.size_deller
+                    if self.particles[i][7]<=0:
+                        if self.size_resize:
+                            if self.return_size:  
+                                if self.particles[i][2]<=self.shape_max_size+10:
+                                    self.particles[i][2]+=self.size_deller
+                            else:
+                                self.particles[i][2]-=self.size_deller
 
                     if self.image_rotate_in_position:
                         center_pos = GL_MATH.Vec2_([self.posx-self.particles[i][0].vector.x,self.posy-self.particles[i][0].vector.y])
@@ -391,6 +469,7 @@ class Particles():
                         
                         #speed_pos = GL_MATH.Vec2_([self.particles[i][1].vector.x+self.particles[i][0].vector.x,self.particles[i][1].vector.y+self.particles[i][0].vector.y])
                         self.particles[i][7]=-int(self.particles[i][0].Angle_to(self.particles[i][1]))+125
+        
         def Xclean(self):
             if self.start_delete:
                 for i in range(len(self.particles)):
@@ -445,7 +524,7 @@ class Particles():
             
             for i in range(len(self.particles)):
                 self.particles[i][0].vector.x+=speed[0]
-                self.particles[i][0].vector.y+=-speed[1]
+                self.particles[i][0].vector.y+=speed[1]
         
         def P(self):
             self.Emiter()
@@ -478,7 +557,7 @@ class Particles():
 ''')
 
     class Bax():
-        def __init__(self,surf,position,max_size,min_size,start_size,size_delta,size_delta2,color,type='c',start_rad=10,rad_delta=0,zikl=False):
+        def __init__(self,surf,position,max_size,min_size,start_size,size_delta,size_delta2,color,type='c',start_rad=10,rad_delta=0,zikl=False,teni=False):
             self.pos = position
             self.max_size = max_size
             self.min_size = min_size
@@ -487,6 +566,7 @@ class Particles():
             self.size_delta = size_delta
             self.size_delta2 = size_delta2
             self.sd2 = self.size_delta2
+            self.teni = teni
 
             self.start_rad = start_rad
             self.sr = self.start_rad
@@ -498,11 +578,15 @@ class Particles():
             self.color = color
             self.type = type
             self.surf = surf
-            
+        def Move(self,speed):
+            self.pos[0]+=speed[0]
+            self.pos[1]+=speed[1]
 
         def Render(self):
             if self.type == 'c':
                 if self.start_rad>0:
+                    if self.teni:
+                        self.surf.GL.Circle('black',[self.pos[0]+4,self.pos[1]+4],self.start_size,self.start_rad,'s','D')
                     self.surf.GL.Circle(self.color,self.pos,self.start_size,self.start_rad,'s','D')
             elif self.type == 'r':
                 if self.start_rad>0:
